/* plone.app.discussion stylesheet */


/* Comments Viewlet
   ---------------------------------------------------------------- */

.comment {
	min-height: 100px;
	overflow: auto;
}

.commentImage {
	width: 90px;
	float: left;
	background: #EEF3F5; /* Todo: Only a temporary solution !!! */
}

.commentImage img {
	   margin-right: 1em;
}

.comment h3,
.comment .documentByLine,
.comment .commentBody {

}

/* Reply Form
   ---------------------------------------------------------------- */

.reply fieldset {
	/* Fix bouncing fieldset when reply form is shown. */
	margin: 0 !important;
}

.reply-to-comment-button {
   /* Hide the reply-to-comment button and show it only if Javascript is enabled.
    * Since the reply functions rely on jQuery.
    */
	display: none;
}

.cancelreplytocomment {
	/* Hide the cancel button in the comment form, show it only in the reply form
	 * in order to close it.
	 */
	display: none;
}

/* Moderation View
   ---------------------------------------------------------------- */

ul.filter {
	margin-top: 2em;
}

.filter form {
	display: inline;
}

.filter input.selected,
.filter input:hover {
	background: #DEE7EC;
}

.filter .firstFormTab input {
    border-left: 1px solid #8CACBB;
}

.filter .lastFormTab input {
    border-right: 1px solid #8CACBB;
}

.filter input {
	background: White;
    border-bottom: 1px solid #8CACBB;
    border-left: 1px dotted #8CACBB;
    border-top: 1px solid #8CACBB;
    margin: 0;
    padding: 0.125em 0.75em;
    text-decoration: none;
}

#bulkactions {
	text-align: left;
}

#dobulkaction {
	margin: 0.3em 0;
}

.comment-publish-button {
    float: left;
}

.comment-delete-button {
    float: right;
}

#review-comments {
	margin: 0;
}

#fieldset-moderate-comments {
	padding: 2em 1em;
}
